@@ -203,32 +203,62 @@ class HDXObject(UserDict, object):
             None
         """
         for field in self.configuration[object_type]['required_fields']:
-            if field not in self.data and field not in ignore_fields:
-                raise HDXError('Field %s is missing in %s!' % (field, object_type))
+            if field not in ignore_fields:
+                if field not in self.data:
+                    raise HDXError('Field %s is missing in %s!' % (field, object_type))
+                if not self.data[field] and not isinstance(self.data[field], bool):
+                    raise HDXError('Field %s is empty in %s!' % (field, object_type))
 
-    def _merge_hdx_update(self, object_type, id_field_name, file_to_upload=None, **kwargs):
-        # type: (str, str, Optional[str], Any) -> None
+    def _hdx_update(self, object_type, id_field_name, file_to_upload=None, force_active=False, **kwargs):
+        # type: (str, str, Optional[str], bool, Any) -> None
         """Helper method to check if HDX object exists and update it
 
         Args:
             object_type (str): Description of HDX object type (for messages)
             id_field_name (str): Name of field containing HDX object identifier
             file_to_upload (Optional[str]): File to upload to HDX
+            force_active (bool): Make object state active. Defaults to False.
             **kwargs: See below
             operation (string): Operation to perform eg. patch. Defaults to update.
+            ignore_field (string): Any field to ignore when checking dataset metadata. Defaults to None.
+
+        Returns:
+            None
+        """
+        if 'batch_mode' in kwargs:  # Whether or not CKAN should change groupings of datasets on /datasets page
+            self.data['batch_mode'] = kwargs['batch_mode']
+        if 'skip_validation' in kwargs:  # Whether or not CKAN should perform validation steps (checking fields present)
+            self.data['skip_validation'] = kwargs['skip_validation']
+        ignore_fields = list()
+        ignore_field = self.configuration['%s' % object_type].get('ignore_on_update')
+        if ignore_field:
+            ignore_fields.append(ignore_field)
+        ignore_field = kwargs.get('ignore_field')
+        if ignore_field:
+            ignore_fields.append(ignore_field)
+        if 'ignore_check' not in kwargs:  # allow ignoring of field checks
+            self.check_required_fields(ignore_fields=ignore_fields)
+        operation = kwargs.get('operation', 'update')
+        self._save_to_hdx(operation, id_field_name, file_to_upload, force_active)
+
+    def _merge_hdx_update(self, object_type, id_field_name, file_to_upload=None, force_active=False, **kwargs):
+        # type: (str, str, Optional[str], bool, Any) -> None
+        """Helper method to check if HDX object exists and update it
+
+        Args:
+            object_type (str): Description of HDX object type (for messages)
+            id_field_name (str): Name of field containing HDX object identifier
+            file_to_upload (Optional[str]): File to upload to HDX
+            force_active (bool): Make object state active. Defaults to False.
+            **kwargs: See below
+            operation (string): Operation to perform eg. patch. Defaults to update.
+            ignore_field (string): Any field to ignore when checking dataset metadata. Defaults to None.
 
         Returns:
             None
         """
         merge_two_dictionaries(self.data, self.old_data)
-        if 'batch_mode' in kwargs:  # Whether or not CKAN should change groupings of datasets on /datasets page
-            self.data['batch_mode'] = kwargs['batch_mode']
-        if 'skip_validation' in kwargs:  # Whether or not CKAN should perform validation steps (checking fields present)
-            self.data['skip_validation'] = kwargs['skip_validation']
-        ignore_field = self.configuration['%s' % object_type].get('ignore_on_update')
-        self.check_required_fields(ignore_fields=[ignore_field])
-        operation = kwargs.get('operation', 'update')
-        self._save_to_hdx(operation, id_field_name, file_to_upload)
+        self._hdx_update(object_type, id_field_name, file_to_upload=file_to_upload, force_active=force_active, **kwargs)
 
     @abc.abstractmethod
     def update_in_hdx(self):
@@ -240,36 +270,38 @@ class HDXObject(UserDict, object):
         """
         raise NotImplementedError
 
-    def _update_in_hdx(self, object_type, id_field_name, file_to_upload=None, **kwargs):
-        # type: (str, str, Optional[str], Any) -> None
+    def _update_in_hdx(self, object_type, id_field_name, file_to_upload=None, force_active=True, **kwargs):
+        # type: (str, str, Optional[str], bool, Any) -> None
         """Helper method to check if HDX object exists in HDX and if so, update it
 
         Args:
             object_type (str): Description of HDX object type (for messages)
             id_field_name (str): Name of field containing HDX object identifier
             file_to_upload (Optional[str]): File to upload to HDX
+            force_active (bool): Make object state active. Defaults to True.
             **kwargs: See below
             operation (string): Operation to perform eg. patch. Defaults to update.
+            ignore_field (string): Any field to ignore when checking dataset metadata. Defaults to None.
 
         Returns:
             None
         """
 
         self._check_load_existing_object(object_type, id_field_name)
-        # We load an existing object even thought it may well have been loaded already
+        # We load an existing object even though it may well have been loaded already
         # to prevent an admittedly unlikely race condition where someone has updated
         # the object in the intervening time
-        self._merge_hdx_update(object_type, id_field_name, file_to_upload, **kwargs)
+        self._merge_hdx_update(object_type, id_field_name, file_to_upload, force_active=force_active, **kwargs)
 
-    def _write_to_hdx(self, action, data, id_field_name, file_to_upload=None):
+    def _write_to_hdx(self, action, data, id_field_name=None, file_to_upload=None):
         # type: (str, Dict, str, Optional[str]) -> Dict
         """Creates or updates an HDX object in HDX and return HDX object metadata dict
 
         Args:
             action (str): Action to perform eg. 'create', 'update'
             data (Dict): Data to write to HDX
-            id_field_name (str): Name of field containing HDX object identifier or None
-            file_to_upload (Optional[str]): File to upload to HDX
+            id_field_name (Optional[str]): Name of field containing HDX object identifier. Defaults to None.
+            file_to_upload (Optional[str]): File to upload to HDX. Defaults to None.
 
         Returns:
             Dict: HDX object metadata
@@ -283,13 +315,17 @@ class HDXObject(UserDict, object):
                 files = None
             return self.configuration.call_remoteckan(self.actions()[action], data, files=files)
         except Exception as e:
-            raisefrom(HDXError, 'Failed when trying to %s %s! (POST)' % (action, data[id_field_name]), e)
+            if id_field_name:
+                idstr = ' %s' % data[id_field_name]
+            else:
+                idstr = ''
+            raisefrom(HDXError, 'Failed when trying to %s%s! (POST)' % (action, idstr), e)
         finally:
             if file_to_upload and file:
                 file.close()
 
-    def _save_to_hdx(self, action, id_field_name, file_to_upload=None):
-        # type: (str, str, Optional[str]) -> None
+    def _save_to_hdx(self, action, id_field_name, file_to_upload=None, force_active=False):
+        # type: (str, str, Optional[str], bool) -> None
         """Creates or updates an HDX object in HDX, saving current data and replacing with returned HDX object data
         from HDX
 
@@ -297,10 +333,13 @@ class HDXObject(UserDict, object):
             action (str): Action to perform: 'create' or 'update'
             id_field_name (str): Name of field containing HDX object identifier
             file_to_upload (Optional[str]): File to upload to HDX
+            force_active (bool): Make object state active. Defaults to False.
 
         Returns:
             None
         """
+        if force_active:
+            self.data['state'] = 'active'
         result = self._write_to_hdx(action, self.data, id_field_name, file_to_upload)
         self.old_data = self.data
         self.data = result
@@ -316,8 +355,8 @@ class HDXObject(UserDict, object):
         raise NotImplementedError
 
     def _create_in_hdx(self, object_type, id_field_name, name_field_name,
-                       file_to_upload=None):
-        # type: (str, str, str, Optional[str]) -> None
+                       file_to_upload=None, force_active=True, **kwargs):
+        # type: (str, str, str, Optional[str], bool, Any) -> None
         """Helper method to check if resource exists in HDX and if so, update it, otherwise create it
 
 
@@ -326,16 +365,18 @@ class HDXObject(UserDict, object):
             id_field_name (str): Name of field containing HDX object identifier
             name_field_name (str): Name of field containing HDX object name
             file_to_upload (Optional[str]): File to upload to HDX (if url not supplied)
+            force_active (bool): Make object state active. Defaults to True.
 
         Returns:
             None
         """
-        self.check_required_fields()
+        if 'ignore_check' not in kwargs:  # allow ignoring of field checks
+            self.check_required_fields()
         if id_field_name in self.data and self._load_from_hdx(object_type, self.data[id_field_name]):
             logger.warning('%s exists. Updating %s' % (object_type, self.data[id_field_name]))
-            self._merge_hdx_update(object_type, id_field_name, file_to_upload)
+            self._merge_hdx_update(object_type, id_field_name, file_to_upload, force_active, **kwargs)
         else:
-            self._save_to_hdx('create', name_field_name, file_to_upload)
+            self._save_to_hdx('create', name_field_name, file_to_upload, force_active)
 
     @abc.abstractmethod
     def delete_from_hdx(self):
@@ -494,41 +535,47 @@ class HDXObject(UserDict, object):
             return list()
         return [x['name'] for x in tags]
 
-    def _add_tag(self, tag):
-        # type: (str) -> bool
+    def _add_tag(self, tag, vocabulary_id=None):
+        # type: (str, Optional[str]) -> bool
         """Add a tag
 
         Args:
             tag (str): Tag to add
+            vocabulary_id (Optional[str]): Vocabulary tag is in. Defaults to None.
 
         Returns:
             bool: True if tag added or False if tag already present
         """
+        tag = tag.lower()
         tags = self.data.get('tags', None)
         if tags:
             if tag in [x['name'] for x in tags]:
                 return False
         else:
             tags = list()
-        tags.append({'name': tag})
+        tagdict = {'name': tag}
+        if vocabulary_id is not None:
+            tagdict['vocabulary_id'] = vocabulary_id
+        tags.append(tagdict)
         self.data['tags'] = tags
         return True
 
-    def _add_tags(self, tags):
-        # type: (List[str]) -> bool
+    def _add_tags(self, tags, vocabulary_id=None):
+        # type: (List[str], Optional[str]) -> List[str]
         """Add a list of tag
 
         Args:
             tags (List[str]): list of tags to add
+            vocabulary_id (Optional[str]): Vocabulary tag is in. Defaults to None.
 
         Returns:
-            bool: True if all tags added or False if any already present.
+            List[str]: Tags that were successfully added
         """
-        alltagsadded = True
+        added_tags = list()
         for tag in tags:
-            if not self._add_tag(tag):
-                alltagsadded = False
-        return alltagsadded
+            if self._add_tag(tag, vocabulary_id=vocabulary_id):
+                added_tags.append(tag)
+        return added_tags
 
     def _get_stringlist_from_commastring(self, field):
         # type: (str) -> List[str]
